From 9687b3fb59a929adcd0ca856406f3b5350791a26 Mon Sep 17 00:00:00 2001
From: Gopu Govindaswamy <gopu@multicorewareinc.com>
Date: Wed, 27 May 2015 15:11:58 +0530
Subject: [PATCH] Dump frame statistics to the csv log file in encode order

 --csv=<fileName> the file name formate is csv, this param option used to
dump following fields into csv file in encoding order

EncodeOrder, SliceType, POC, QP, Bytes, Y PSNR, U PSNR, V PSNR, YUV PSNR,
SSIM and SSIM(dB)
---
 common/common.c   |  2 ++
 common/common.h   |  3 +++
 encoder/encoder.c | 60 +++++++++++++++++++++++++++++++++++++++++++++++++++++++
 x264.c            |  6 +++++-
 x264.h            |  5 ++++-
 5 files changed, 74 insertions(+), 2 deletions(-)

diff --git a/common/common.c b/common/common.c
index de5d441..fddd354 100644
--- a/common/common.c
+++ b/common/common.c
@@ -1048,6 +1048,8 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
         p->psz_clbin_file = strdup( value );
     OPT("opencl-device")
         p->i_opencl_device = atoi( value );
+    OPT("csv")
+        p->csvfn = strdup( value );
     else
         return X264_PARAM_BAD_NAME;
 #undef OPT
diff --git a/common/common.h b/common/common.h
index 7d5e5fb..59711c7 100644
--- a/common/common.h
+++ b/common/common.h
@@ -969,6 +969,9 @@ struct x264_t
 
     x264_lookahead_t *lookahead;
 
+    /* csv log file handler */
+    FILE *csvfh;
+
 #if HAVE_OPENCL
     x264_opencl_t opencl;
 #endif
diff --git a/encoder/encoder.c b/encoder/encoder.c
index edc387c..c5ea10d 100644
--- a/encoder/encoder.c
+++ b/encoder/encoder.c
@@ -1627,6 +1627,32 @@ x264_t *x264_encoder_open( x264_param_t *param )
     }
 #endif
 
+    /* open CSV file handle for dump log information into csv file */
+    if( h->param.csvfn )
+    {
+        h->csvfh = x264_fopen( h->param.csvfn, "r" );
+        if( h->csvfh )
+        {
+            fclose( h->csvfh );
+            /* file already exist re-open the file with append mode */
+            h->csvfh = x264_fopen( h->param.csvfn, "ab" );
+            fputs( "\n", h->csvfh );
+        }
+        else
+        {
+            /* open new csv file and write header */
+            h->csvfh = x264_fopen( h->param.csvfn, "wb" );
+            if( h->csvfh )
+                fprintf( h->csvfh, "EncodeOrder, Type, POC, QP, Bytes, Y PSNR, U PSNR, V PSNR, YUV PSNR, SSIM, SSIM(dB)\n" );
+        }
+        if( !h->csvfh )
+        {
+            x264_log( h, X264_LOG_ERROR, "Unable to open CSV log file <%s>, aborting\n", h->param.csvfn );
+            goto fail;
+        }
+    }
+
+
     h->thread[0] = h;
     for( int i = 1; i < h->param.i_threads + !!h->param.i_sync_lookahead; i++ )
         CHECKED_MALLOC( h->thread[i], sizeof(x264_t) );
@@ -1666,6 +1692,9 @@ x264_t *x264_encoder_open( x264_param_t *param )
 
         if( allocate_threadlocal_data && x264_macroblock_cache_allocate( h->thread[i] ) < 0 )
             goto fail;
+
+        // share same csv log file handler for all threads 
+        h->thread[i]->csvfh = h->csvfh;
     }
 
 #if HAVE_OPENCL
@@ -3966,6 +3995,34 @@ static int x264_encoder_frame_end( x264_t *h, x264_t *thread_current,
               frame_size,
               psz_message );
 
+    // per frame CSV logging if the file handler is valid
+    if( h->csvfh )
+    {
+        fprintf( h->csvfh, "%4d, %c, %3d, %.2f, %d, ",
+                 h->i_frame,
+                 h->sh.i_type == SLICE_TYPE_I ? 'I' : ( h->sh.i_type == SLICE_TYPE_P ? 'P' : 'B' ),
+                 h->fdec->i_poc,
+                 h->fdec->f_qp_avg_aq,
+                 frame_size );
+
+        if( h->param.analyse.b_psnr )
+            fprintf( h->csvfh, "%5.2f, %5.2f, %5.2f, %5.2f, ",
+                     pic_out->prop.f_psnr[0],
+                     pic_out->prop.f_psnr[1],
+                     pic_out->prop.f_psnr[2],
+                     pic_out->prop.f_psnr_avg );
+        else
+            fputs( "-, -, -, -, ", h->csvfh );
+        if( h->param.analyse.b_ssim )
+            fprintf( h->csvfh, "%.5f, %5.3f",
+                     pic_out->prop.f_ssim,
+                     x264_ssim(pic_out->prop.f_ssim) );
+        else
+            fputs( "-, -", h->csvfh );
+        fputs( "\n", h->csvfh );
+    }
+
+
     // keep stats all in one place
     x264_thread_sync_stat( h->thread[0], h );
     // for the use of the next frame
@@ -4344,6 +4401,9 @@ void    x264_encoder_close  ( x264_t *h )
 
     h = h->thread[0];
 
+    if( h->csvfh )
+        fclose( h->csvfh );
+
     for( int i = 0; i < h->i_thread_frames; i++ )
         if( h->thread[i]->b_thread_active )
             for( int j = 0; j < h->thread[i]->i_ref[0]; j++ )
diff --git a/x264.c b/x264.c
index 4dd2a26..f7f217f 100644
--- a/x264.c
+++ b/x264.c
@@ -937,6 +937,8 @@ static void help( x264_param_t *defaults, int longhelp )
     H0( "      Filter options may be specified in <filter>:<option>=<value> format.\n" );
     H0( "\n" );
     H0( "      Available filters:\n" );
+    H0(" \n ");
+    H1("       --csv <string>          Comma separated log file(csv file) per frame \n" );
     x264_register_vid_filters();
     x264_vid_filter_help( longhelp );
     H0( "\n" );
@@ -974,7 +976,8 @@ typedef enum
     OPT_DTS_COMPRESSION,
     OPT_OUTPUT_CSP,
     OPT_INPUT_RANGE,
-    OPT_RANGE
+    OPT_RANGE,
+    OPT_CSVFILE
 } OptionsOPT;
 
 static char short_options[] = "8A:B:b:f:hI:i:m:o:p:q:r:t:Vvw";
@@ -1143,6 +1146,7 @@ static struct option long_options[] =
     { "input-range", required_argument, NULL, OPT_INPUT_RANGE },
     { "stitchable",        no_argument, NULL, 0 },
     { "filler",            no_argument, NULL, 0 },
+    { "csv",         required_argument, NULL, OPT_CSVFILE },
     {0, 0, 0, 0}
 };
 
diff --git a/x264.h b/x264.h
index 7cc5fcc..8d71d50 100644
--- a/x264.h
+++ b/x264.h
@@ -41,7 +41,7 @@
 
 #include "x264_config.h"
 
-#define X264_BUILD 146
+#define X264_BUILD 147
 
 /* Application developers planning to link against a shared library version of
  * libx264 from a Microsoft Visual Studio or similar development environment
@@ -502,6 +502,9 @@ typedef struct x264_param_t
     int i_slice_count_max;   /* Absolute cap on slices per frame; stops applying slice-max-size
                               * and slice-max-mbs if this is reached. */
 
+    /* filename of CSV log the encoder will emit per-slice statistics to this log file in encode order */
+    const char *csvfn;
+
     /* Optional callback for freeing this x264_param_t when it is done being used.
      * Only used when the x264_param_t sits in memory for an indefinite period of time,
      * i.e. when an x264_param_t is passed to x264_t in an x264_picture_t or in zones.
-- 
1.9.5.msysgit.1

